import AlteryxPythonSDK as Sdk
import json
import datetime
import decimal

AUTHORIZATION_SERVER_URL = 'https://login.microsoftonline.com'
BASE_API_ENDPOINT = 'https://api.powerbi.com/v1.0/myorg'
RESOURCE_URI='https://analysis.windows.net/powerbi/api'

OUTPUT_MODE_APPEND_TO_TABLE_IF_EXISTS = 'OUTPUT_MODE_APPEND_TO_TABLE_IF_EXISTS';
OUTPUT_MODE_RENAME_DATASET_IF_EXISTS = 'OUTPUT_MODE_RENAME_DATASET_IF_EXISTS';
OUTPUT_MODE_REPLACE_DATASET_IF_EXISTS = 'OUTPUT_MODE_REPLACE_DATASET_IF_EXISTS';

OUTPUT_MODES = {
    OUTPUT_MODE_REPLACE_DATASET_IF_EXISTS,
    OUTPUT_MODE_RENAME_DATASET_IF_EXISTS,
    OUTPUT_MODE_APPEND_TO_TABLE_IF_EXISTS
}

class FailToAuthenticateOnTokenExpiration(Exception):
    pass

def get_api_url(endpoint):
    return '{}/{}'.format(BASE_API_ENDPOINT, endpoint)

def get_power_bi_type(sdk_type):
    try:
        return {
            Sdk.FieldType.bool: 'Boolean',
            Sdk.FieldType.v_wstring: 'String',
            Sdk.FieldType.v_string: 'String',
            Sdk.FieldType.wstring: 'String',
            Sdk.FieldType.string: 'String',
            Sdk.FieldType.blob: 'String',
            Sdk.FieldType.int64: 'Int64',
            Sdk.FieldType.int32: 'Int64',
            Sdk.FieldType.int16: 'Int64',
            Sdk.FieldType.byte: 'Int64',
            Sdk.FieldType.float: 'Double',
            Sdk.FieldType.double: 'Double',
            Sdk.FieldType.fixeddecimal: 'Decimal',
            Sdk.FieldType.date: 'DateTime',
            Sdk.FieldType.time: 'DateTime',
            Sdk.FieldType.datetime: 'DateTime',
        }[sdk_type]
    except KeyError as e:
        return 'String'

def serialize_datetime(obj):
    if isinstance(obj, (datetime.datetime, decimal.Decimal)):
        return obj.__str__()
    elif isinstance(obj, datetime.timedelta):
        return (datetime.datetime(1970, 1, 1) + obj).__str__()

def get_table_schema(dataset_name, table_name, record_info):
    return {
        "name": table_name,
        "columns": [{
                "name": sdk_field.name,
                "dataType": get_power_bi_type(sdk_field.type)
            } for sdk_field in record_info]
    }
